/*
 * Copyright (C) 2012 Michael Andersen <michael@steelcode.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.steelcode.pamflit;

import java.util.ArrayList;
import java.util.Arrays;

/**
 *
 * @author Michael Andersen <michael@steelcode.com>
 */
public class PAMflit
{

    /*
     * Assembles and programs the given file
     */
    public static int programFile(String filename)
    {
        ArrayList<Integer> codes = PVMParser.parse(filename);
        //for (int i : codes) System.err.print(i +",");
        ProgramImage pi = new ProgramImage();
        pi.writeCodes(0, codes);
        System.out.println("Attempting to program device");
        int rv = pi.programToDevice();
        if(rv == 0)
        {
            System.out.println("Device successfully programmed");
        }
        else
        {
            System.out.println("Failed to program device: " +( 
                    (rv == -1)?"could not connect":
                    (rv == -2)?"could not erase buffer":
                    (rv == -3)?"could not fill buffer":
                    (rv == -4)?"could not initiate flashing":
                    "unknown error"));
            
        }
        return rv;
    }
    public static void usage()
    {
        System.out.println("PAM Firmware Loader and Interactive Toolkit");
        System.out.println(" arguments: (program | assemble-program | program-connect) filename");
    }
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args)
    {
        NativeLink.initNatives();
        
        if ((args.length == 1 && !args[0].equalsIgnoreCase("connect")) ||
            args.length == 0 || args.length > 2)
        {
            usage();
            System.exit(1);
        }
        //I had java 7 switch-on-strings here and it was good. But alas it was
        //not meant to be
        if (args[0].equalsIgnoreCase("assemble"))
        {
            PVMParser.parse(args[1]);
        } 
        else if (args[0].equalsIgnoreCase("program"))
        {
            programFile(args[1]);
        }
        else if (args[0].equalsIgnoreCase("program-connect"))
        {
            if(programFile(args[1]) == 0)
            {
                InteractiveConsole ic = new InteractiveConsole();
                ic.startBlocking();
            }
        }
        else if (args[0].equalsIgnoreCase("connect"))
        {
            InteractiveConsole ic = new InteractiveConsole();
            ic.startBlocking();
        }
        else
        {
            usage();
            System.exit(1);
        }
    }
}
